
//paramaters
static Double_t dpxoverp, dpyoverp, dpzoverp, dele, estrag, multi_theta;
static Int_t thrmethod;
static Double_t thr, treso, attlength, vl;
static Int_t rotation;
static Double_t length, vlength, width, depth, gap, zdepth, zdepth_back, shift;
static Int_t multihit;
static Double_t aminmin, ein, sp;
static Double_t step;

int ReadParamaters(std::string facility, std::string detector,
		   std::string fragment, std::string target)
{
  CXmlManager xml;
  if (!xml.OpenFile("macros/analysis/Analysis.xml")) return 0;

  if (!xml.GetValue("//"+facility+"//dpxoverp", &dpxoverp)) return 0;
  if (!xml.GetValue("//"+facility+"//dpyoverp", &dpyoverp)) return 0;
  if (!xml.GetValue("//"+facility+"//dpzoverp", &dpzoverp)) return 0;

  if (!xml.GetValue("//"+fragment+"/"+target+"/dele", &dele)) return 0;
  if (!xml.GetValue("//"+fragment+"/"+target+"/estrag", &estrag)) return 0;
  if (!xml.GetValue("//"+fragment+"/"+target+"/scatter", &multi_theta)) return 0;

  if (!xml.GetValue("//"+detector+"//thrmethod", &thrmethod)) return 0;
  if (!xml.GetValue("//"+detector+"//thr", &thr)) return 0;
  if (!xml.GetValue("//"+detector+"//treso", &treso)) return 0;
  if (!xml.GetValue("//"+detector+"//attlength", &attlength)) return 0;
  if (!xml.GetValue("//"+detector+"//vl", &vl)) return 0;
  if (!xml.GetValue("//"+detector+"//rotation", &rotation)) return 0;
  if (!xml.GetValue("//"+detector+"//length", &length)) return 0;
  if (!xml.GetValue("//"+detector+"//vlength", &vlength)) return 0;
  if (!xml.GetValue("//"+detector+"//width", &width)) return 0;
  if (!xml.GetValue("//"+detector+"//depth", &depth)) return 0;
  if (!xml.GetValue("//"+detector+"//gap", &gap)) return 0;
  if (!xml.GetValue("//"+detector+"//zdepth", &zdepth)) return 0;
  if (!xml.GetValue("//"+detector+"//zdepth_back", &zdepth_back)) return 0;
  if (!xml.GetValue("//"+detector+"//shift", &shift)) return 0;
  if (!xml.GetValue("//"+detector+"//multihit", &multihit)) return 0;
  if (!xml.GetValue("//"+detector+"//aminmin", &aminmin)) return 0;
  if (!xml.GetValue("//"+detector+"//ein", &ein)) return 0;
  if (!xml.GetValue("//"+detector+"//sp", &sp)) return 0;
  if (!xml.GetValue("//"+detector+"//step", &step)) return 0;

  return 1;
}

void SimToExp(CDataHimeSim* simdata, CDataHime* dataexp, std::string method)
{
  std::deque<CDataHimeSim::OneData>::iterator itsim;
  itsim = simdata->fDequeOfOneData.begin();
  while(itsim != simdata->fDequeOfOneData.end()){
    std::vector<Int_t> id;
    id.push_back(itsim->fIdMother);
    id.push_back(itsim->fId);

    Double_t xl = length/2.+itsim->fPosition.X()-shift;
    Double_t xr = length/2.-itsim->fPosition.X()+shift;
    Double_t yl = length/2.+itsim->fPosition.Y();
    Double_t yr = length/2.-itsim->fPosition.Y();

    Double_t fEdepee = CKinema::MeVtoMeVee(itsim->fKineticEnergy,
					   itsim->fKineticEnergy - itsim->fEdep,
					   itsim->fParticleName.Data());

    Double_t tl, tr, al, ar;
    if(rotation == 1){
      if(id[0]%2){
	tl = itsim->fTime + yl/vl;
	tr = itsim->fTime + yr/vl;
	al = fEdepee*exp(-yl/attlength);
	ar = fEdepee*exp(-yr/attlength);
      }else{
	tl = itsim->fTime + xl/vl;
	tr = itsim->fTime + xr/vl;
	al = fEdepee*exp(-xl/attlength);
	ar = fEdepee*exp(-xr/attlength);
      }
    }else{
      tl = itsim->fTime + xl/vl;
      tr = itsim->fTime + xr/vl;
      al = fEdepee*exp(-xl/attlength);
      ar = fEdepee*exp(-xr/attlength);
    }      

    CDataHime::OneData* onedata = dataexp->Find(id[0], id[1]);
    if(onedata){
      if(onedata->fTL > tl) onedata->fTL = tl;
      if(onedata->fTR > tr) onedata->fTR = tr;
      onedata->fAL += al;
      onedata->fAR += ar;
    }else{
      dataexp->fDequeOfOneData.push_back(CDataHime::OneData(id, tl, tr, al, ar));
    }

    ++itsim;
  }

  std::deque<CDataHime::OneData>::iterator it;
  it = dataexp->fDequeOfOneData.begin();
  while(it != dataexp->fDequeOfOneData.end()){

    //thr判定その1
    if(thrmethod == 1){
      if(it->fAL < thr || it->fAR < thr){
	it = dataexp->fDequeOfOneData.erase(it);
	continue;
      }
    }

    it->fTL += gRandom->Gaus(0,treso);
    it->fTR += gRandom->Gaus(0,treso);  
    
    it->fT = (it->fTL + it->fTR - length/vl)/2;
    it->fE = sqrt(it->fAL*it->fAR)*exp(length/(2.*attlength));
    
    //thr判定その2
    if(thrmethod == 2){
      if(it->fE < thr){
	it = dataexp->fDequeOfOneData.erase(it);
	continue;
      }
    }

    Double_t x, y, z;
    if(rotation == 1){
      if(it->fId[0]%2){
	x = it->fId[1]*width + width/2. - vlength/2. + shift;
	y = vl*(it->fTL-it->fTR)/2.;
      }else{
	x = vl*(it->fTL-it->fTR)/2. + shift;
	y = it->fId[1]*width + width/2. - vlength/2.;
      }
    }else{
      x = vl*(it->fTL-it->fTR)/2. + shift;
      y = it->fId[1]*width + width/2. - vlength/2.;
    }
    z = it->fId[0]*(depth+gap) + zdepth + depth/2.;

    it->fX.SetXYZ(x, y, z);

    ++it;
  }

  if(method == "Track"){
    Track(dataexp);
  }else if(method == "Simple"){
    Simple(dataexp);
  }else if(method == "Niren"){
    Niren(dataexp);
  }else{
    std::cerr << "method:" << method << " cannot use" << std::endl;
    return;
  }
}

void Track(CDataHime* dataexp)
{
#ifdef VISUALIZE
  TGraph2DErrors* graph = new TGraph2DErrors;//staticを付けるとメモリの再確保がなくなり早い
#endif

  if(dataexp->fDequeOfOneData.empty()) return;

  //どうせ何回も使うので
  static std::deque<CDataHime::OneData>::iterator it;
  static std::deque<CDataHime::OneData>::iterator itmin;

  //最速イベントではなく先頭イベント。
  //最速を選ぶと時間分解能の分だけ微妙にシフトすると思われる。
  itmin = dataexp->fDequeOfOneData.begin();
  it = dataexp->fDequeOfOneData.begin();
  while(it != dataexp->fDequeOfOneData.end()){
    if(it->fX.Z() < itmin->fX.Z()) itmin = it;
    ++it;
  }

  //遅いイベントは関係ないので切り捨て。
  it = dataexp->fDequeOfOneData.begin();
  double tmin = itmin->fT;//roop内部でeraseを使ってるので途中でitminが不正になりうる。
  while(it != dataexp->fDequeOfOneData.end()){
    //先頭でhitしてからある程度時間がかかったeventは切ってしまう
    //時間で排除する代わりに、シグナルの近傍度みたいな量を定義して、
    //それでcutしたほうがノイズを効率的に落とせるかも
    if(abs(it->fT - tmin) > 1.2){
      it = dataexp->fDequeOfOneData.erase(it);
      continue;
    }

    ++it;
  }

  //eraseしたので最速イベントのイテレータを取り直す。(必須!!)
  itmin = dataexp->fDequeOfOneData.begin();
  it = dataexp->fDequeOfOneData.begin();
  while(it != dataexp->fDequeOfOneData.end()){
    if(it->fX.Z() < itmin->fX.Z()) itmin = it;
    ++it;
  }

  //hit数に満たない場合はcut
  if((int)dataexp->fDequeOfOneData.size() < multihit){
    dataexp->fEventFlag = 0;//in-validなデータの場合は0
    return;
  }

  //2Dfit(zの誤差を無視する)でtracking
  Int_t point = 0;
  std::vector<double> pointx, pointy, pointz;
  std::vector<double> pointxe, pointye, pointze;
  it = dataexp->fDequeOfOneData.begin();
  while(it != dataexp->fDequeOfOneData.end()){
    pointx.push_back(it->fX.X());
    pointy.push_back(it->fX.Y());
    pointz.push_back(it->fX.Z());
    if(it->fId[0]%2){
      pointxe.push_back(width/sqrt(12));
      pointye.push_back(vl*treso/sqrt(2));
    }else{	 
      pointxe.push_back(vl*treso/sqrt(2));
      pointye.push_back(width/sqrt(12));
    }
    pointze.push_back(depth/sqrt(12));

#ifdef VISUALIZE
    graph->SetPoint(point, it->fX.X(), it->fX.Y(), it->fX.Z());
    if(it->fId[0]%2){
      graph->SetPointError(point, width/sqrt(12), vl*treso/sqrt(2), depth/sqrt(12));
    }else{	 
      graph->SetPointError(point, vl*treso/sqrt(2), width/sqrt(12), depth/sqrt(12));
    }
#endif

    ++point;
    ++it;
  }
  double parFit[4];

  CAnalysis::LinearFit(pointz, pointx, pointxe, parFit[0], parFit[1]);
  CAnalysis::LinearFit(pointz, pointy, pointye, parFit[2], parFit[3]);
  double aminx = CAnalysis::SD(pointz, pointx, parFit[0], parFit[1]);
  double aminy = CAnalysis::SD(pointz, pointy, parFit[2], parFit[3]);
  double amin = sqrt(aminx*aminx+aminy*aminy);//rmsっぽいもの

  //rmsを収集
  TH1D* hamin;
  gDirectory->GetObject("rms", hamin);
  hamin->Fill(amin);

  //理想的には選択的にトラックを引くアルゴリズムが必要。
  //たとえばamin<5になるまで、計算に入れる点を変えるとか。
  //今はaminminで単純に切り捨てごめん。
  if(amin > aminmin){
    dataexp->fEventFlag = 0;//in-validなデータの場合は0
    return;
  }

  //analysis of timing
  TVector3 xf(parFit[1], parFit[3], 1.);//fitした直線にそったベクトル
  double theta = xf.Angle(itmin->fX);

  // derive recoile proton energy from theta(lab)
  CParticle pin = CParticleDefinition::GetParticle("proton");
  CParticle nin = CParticleDefinition::GetParticle("neutron");
  CParticle pout = CParticleDefinition::GetParticle("proton");
  CParticle nout = CParticleDefinition::GetParticle("neutron");

  nin.k(ein-sp);//一陽子分離エネルギーを考慮。ほとんど効かないけど。
  pin.k(0);//Fermi motionは無視して静止していると考える。

  //lab系のthetaをパラメータとして、剛体の2体散乱をさせる。
  Double_t theta_cm = CKinema::Scatter(theta, 0., nin, pin, pout, nout);

  //thetaを取得
  TH1D* htheta_exp;
  gDirectory->GetObject("theta_exp", htheta_exp);
  htheta_exp->Fill(theta);
  TH1D* htheta_cm_exp;
  gDirectory->GetObject("theta_cm_exp", htheta_cm_exp);
  htheta_cm_exp->Fill(theta_cm);

  //timingの解析の本体
  double timing = 0;
  int count = 0;
  it = dataexp->fDequeOfOneData.begin();
  while(it != dataexp->fDequeOfOneData.end()){
    double tsum = -1.;
    if(it == itmin){
      tsum = 0;
      //先頭でhitしてから4層通過分程度時間がかかったeventは時間の計算には参加させない
    }else if(abs(it->fT - itmin->fT) < 0.8){
      double xtemp, ytemp, ztemp;
      if(it->fId[0]-itmin->fId[0] > 0){
	//二層め以降で成ったシグナルは荷電粒子によるものと思われ、
	//シミュレーション的にはシンチレータの端面で鳴るのでZを補正。
	//実験でも基本的に同じ補正でいいはずだが未知
	CAnalysis::line(it->fX.Z()-depth/2., parFit, xtemp, ytemp, ztemp);
      }else{
	CAnalysis::line(it->fX.Z(), parFit, xtemp, ytemp, ztemp);	
      }
      TVector3 X(xtemp, ytemp, ztemp);
      CAnalysis::line(itmin->fX.Z(), parFit, xtemp, ytemp, ztemp);
      TVector3 Xmin(xtemp, ytemp, ztemp);
      TVector3 distance_v = X - Xmin;
      double distance = distance_v.Mag();

      CParticle ptemp = pout;
      tsum  = CKinema::BetheBloch::TimeOut
	("p", "Scint", ptemp,
	 distance - (it->fId[0]-itmin->fId[0])*gap/cos(distance_v.Theta()), step); //空間の分を引く
      if(tsum >= 0.){
	//ほんとは2cm+2cm+2cm+....と交互に時間を足していかなければならない。
	//ここで少しmeanのずれが生じていると考えられる。
	tsum += (distance/2.)/((pout.beta()+ptemp.beta())*CKinema::fLightVelocity/2.);
      }
    }
    if(tsum >= 0){
      //補正した結果、改善する方向に向かわない場合は補正に参加させない
      if(abs((it->fT - tsum) - itmin->fT) < treso*2){
	timing += it->fT - tsum;
	++count;
      }
    }
    ++it;
  }
  timing /= count;

  //位置はlinearfitで求まった先頭、時間は上の解析で補正した時間
  double xtemp, ytemp, ztemp;
  CAnalysis::line(itmin->fX.Z(),parFit,xtemp,ytemp,ztemp);
  dataexp->fX.SetXYZT(xtemp, ytemp, ztemp, timing);

  //X,Y,Z,Tが求まったらMomentumを計算しておく。
  //この関数の内部で非数が渡されてないかエラーチェックが走る
  dataexp->CalMomentum();

#ifdef VISUALIZE
  graph->SetMarkerStyle(8);
  graph->SetMarkerSize(1);
  graph->GetXaxis()->SetLimits(-length/2., length/2.);
  graph->GetYaxis()->SetLimits(-length/2., length/2.);
  graph->SetMinimum(zdepth);
  graph->SetMaximum(zdepth+zdepth_back);

  if(0){
    graph->Draw("PERR");
  }else{
    static bool init = true;
    if(init){
      graph->Draw("PERR");
      init = false;
    }else{
      graph->Draw("PERRSAME");
    }
  }

  CAnalysis::DrawLine(parFit, kRed, zdepth, zdepth+zdepth_back);

  TCanvas* canvas = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("canvas");
  canvas->Update();

  CAnalysis::EventLoop();
#endif

  dataexp->fEventFlag = 1;//validなデータの場合は1

  return;
}

void Simple(CDataHime* dataexp)
{
  static std::deque<CDataHime::OneData>::iterator it;
  static std::deque<CDataHime::OneData>::iterator itmin;

  if(dataexp->fDequeOfOneData.empty()){
    dataexp->fEventFlag = 0;
    return;
  }

  itmin = dataexp->fDequeOfOneData.begin();
  it = dataexp->fDequeOfOneData.begin();
  while(it != dataexp->fDequeOfOneData.end()){
    if(itmin->fX.Z() > it->fX.Z()) itmin = it;
    ++it;
  }

  dataexp->fX.SetXYZT(itmin->fX.X(), itmin->fX.Y(), itmin->fX.Z(), itmin->fT);

  dataexp->CalMomentum();

  dataexp->fEventFlag = 1;

  return;
}

void Niren(CDataHime* dataexp)
{
  static std::deque<CDataHime::OneData>::iterator it;
  static std::deque<CDataHime::OneData>::iterator itmin;

  if(dataexp->fDequeOfOneData.empty()){
    dataexp->fEventFlag = 0;
    return;
  }

  itmin = dataexp->fDequeOfOneData.begin();
  it = dataexp->fDequeOfOneData.begin();
  while(it != dataexp->fDequeOfOneData.end()){
    if(itmin->fX.Z() > it->fX.Z()) itmin = it;
    ++it;
  }

  //ここのアルゴリズムがかなり適当なのでだめだめ。
  //本気で2連hitを使うならdtから得たyとの整合性を取ったりしないとだめすぎる。
  Double_t zmin = itmin->fX.Z() - zdepth+1;
  Int_t idmin = zmin/2./depth+0.1;
  std::deque<CDataHime::OneData>::iterator itmin2;
  itmin2 = dataexp->fDequeOfOneData.begin();
  it = dataexp->fDequeOfOneData.begin();
  while(it != dataexp->fDequeOfOneData.end()){
    Double_t z = it->fX.Z() - (zdepth+1);
    Int_t id = z/2./depth+0.1;
    if(idmin+1 == id){
      itmin2 = it;
      break;
    }
    ++it;
  }
  
  if(itmin2 == dataexp->fDequeOfOneData.begin()){
    dataexp->fEventFlag = 0;
    return;
  }
  
  if(idmin%2){
    dataexp->fX.SetX(itmin->fX.X());
    dataexp->fX.SetY(itmin2->fX.Y());
  }else{
    dataexp->fX.SetX(itmin2->fX.X());
    dataexp->fX.SetY(itmin->fX.Y());
  }
  dataexp->fX.SetZ(itmin->fX.Z());

// ノイズが多すぎてうまく行かない
//  Double_t meanbeta = 0.6;
//  Double_t meanlength = 4.;
//  Double_t time;
//  if(itmin->fX.Z() - itmin2->fX.Z() < 1.){
//    time = (itmin->fT + itmin2->fT)/2.;
//  }else{    
//    time = (itmin->fT + itmin2->fT)/2. - meanlength/(meanbeta*CKinema::fLightVelocity);
//  }
//  dataexp->fX.SetT(time);

  //シンプルに先頭の時間
  dataexp->fX.SetT(itmin->fT);

  dataexp->CalMomentum();

  dataexp->fEventFlag = 1;

  return;
}

// independent functions

int GetFirstProtonInformation(CDataHimeSim* data, CParticle& proton)
{
  static std::deque<CDataHimeSim::OneData>::iterator itsim;
  static std::deque<CDataHimeSim::OneData>::iterator itmin;
  itsim = (data->fDequeOfOneData).begin();
  itmin = (data->fDequeOfOneData).end();
  Double_t tmin = 1e+10;
  while(itsim != data->fDequeOfOneData.end()){
    if(itsim->fParticleName == "proton"){
      if(tmin > itsim->fTime){
	tmin = itsim->fTime;
	itmin = itsim;
      }
    }
    ++itsim;
  }

  if(itmin == data->fDequeOfOneData.end()){
    return 0;
  }else{
    proton.p(itmin->fMomentum.x(),
	     itmin->fMomentum.y(),
	     itmin->fMomentum.z());

    return 1;
  }
}

double GetTotalEnergyLoss(CDataHime* data)
{
  static std::deque<CDataHime::OneData>::iterator it;
  it = (data->fDequeOfOneData).begin();
  double energy=0.;

  while(it != data->fDequeOfOneData.end()){
    energy += it->fE;
    ++it;
  }

  return energy;
}

void FoldChargedParticle(TVector3 momentum, CParticle& particle,
			 TVector3 dpoverp, double dele, double estrag, double multi_theta)
{
  static TRandom3 random;

  particle.p(momentum.Px(),
	     momentum.Py(),
	     momentum.Pz());

  double detodp_coef = (particle.k()+particle.m())/(particle.k()+2*particle.m())
    /particle.k()*particle.p();
  double sigma[] = {momentum.Mag()*dpoverp.x(),
		    momentum.Mag()*dpoverp.y(),
		    momentum.Mag()*dpoverp.z(),
		    dele*detodp_coef,
		    estrag*detodp_coef,
		    multi_theta*momentum.Pz()};

  //energy loss difference
  momentum.SetZ(momentum.Pz()+random.Uniform(-sigma[3]/2., sigma[3]/2.));
  //multiple scattering and energy stragglling
  momentum.SetXYZ(random.Gaus(momentum.Px(), sigma[5]),
		  random.Gaus(momentum.Py(), sigma[5]),
		  random.Gaus(momentum.Pz(), sigma[4]));
  //detector resolution
  particle.p(random.Gaus(momentum.Px(), sigma[0]),
	     random.Gaus(momentum.Py(), sigma[1]),
	     random.Gaus(momentum.Pz(), sigma[2]));
}
