
#include "TFile.h"
#include "TTree.h"
#include "TCanvas.h"
#include "TH1D.h"
#include "TF1.h"
#include "TVector3.h"

#include <iostream>
#include <string>
#include <cmath>

#include "CDataHimeSim.hh"
#include "CKinema.hh"
#include "CParticle.hh"
#include "CParticleDefinition.hh"

void Angle(std::string filename, std::string element)
{
  TCanvas* canvas = new TCanvas("canvas","",200,10,800,400);
  std::deque<TVector3> momentum;

  CParticle neutron = CParticleDefinition::GetParticle("neutron");
  CParticle proton = CParticleDefinition::GetParticle("proton");
  CParticle carbon = CParticleDefinition::GetParticle("12C");

  {
    TFile* file = TFile::Open(filename.c_str(), "READ");
    if(!file) return;

    TTree* tree;
    file->GetObject("Koyomi",tree);
    tree->Print();

    TFile* workfile = TFile::Open("root/temp.root","recreate");
    if(!workfile) return;

    TH1D* hplab = new TH1D("hplab", "theta", 100, 0., TMath::Pi());
    TH1D* hpcm = new TH1D("hpcm", "theta", 100, 0., TMath::Pi());

    CDataHimeSim* data = 0;
    TBranch *bdata = 0;
    if(tree->SetBranchAddress("hime",&data,&bdata)
       == TTree::kMissingBranch) return;

    Long64_t nentries = tree->GetEntriesFast();
    for(Long64_t i=0; i<nentries; i++) {
      if(i%100000 == 0) std::cout << "\r event:" << i << std::flush;

      Long64_t entry = tree->LoadTree(i);
      bdata->GetEntry(entry);

      static std::deque<CDataHimeSim::OneData>::iterator itsim;
      static std::deque<CDataHimeSim::OneData>::iterator itmin;
      itsim = data->fDequeOfOneData.begin();
      itmin = data->fDequeOfOneData.end();
      Double_t tmin = 1e+10;
      while(itsim != data->fDequeOfOneData.end()){
	if(itsim->fParticleName == "proton"){
	  if(tmin > itsim->fTime){
	    tmin = itsim->fTime;
	    itmin = itsim;
	  }
	}
	++itsim;
      }

      if(itmin != data->fDequeOfOneData.end()){
	neutron.p(data->fMomentum.x(),data->fMomentum.y(),data->fMomentum.z());
	proton.p(0, 0, 0);
	carbon.p(0, 0, 0);
	Double_t beta;
	if(element == "H"){
	  beta = (neutron.p()+proton.p())/(neutron.e()+proton.e());
	}else if(element == "C"){
	  beta = (neutron.p()+carbon.p())/(neutron.e()+carbon.e());
	}else{
	  return;
	}

	proton.p(itmin->fMomentum.x(),
		 itmin->fMomentum.y(),
		 itmin->fMomentum.z());
	CKinema::LorentzBoost(proton, beta, 2);

	hplab->Fill(itmin->fMomentum.Theta());
	hpcm->Fill(proton.theta());
      }
    }
    std::cout << std::endl;

    TH1D* hplabx = new TH1D(*hplab);
    TH1D* hpcmx = new TH1D(*hpcm);

    TF1* fsin = new TF1("fsin", "1/sin(x)",0,4);
    hplabx->Multiply(fsin);
    hpcmx->Multiply(fsin);

    canvas->Divide(2,1);
    canvas->cd(1);
    hplabx->Draw();
    canvas->cd(2);
    hpcmx->Draw();

    canvas->Update();
  }

  std::string::iterator it = filename.begin();
  while(it != filename.end()){
    if(*it == '.') break;
    ++it;
  }
  filename.replace(it, filename.end(), ".eps");
  canvas->Print(filename.c_str());
}


