   
#include <TMath.h>
#include <TGraph2D.h>
#include <TRandom2.h>
#include <TStyle.h>
#include <TCanvas.h>
#include <TF2.h>
#include <TH1.h>
#include <TVirtualFitter.h>
#include <TPolyLine3D.h>
#include <Math/Vector3D.h>

#include "TROOT.h"
#include "TFile.h"
#include "TTree.h"
#include "TCanvas.h"
#include "TH2D.h"
#include "TH1F.h"
#include "TH3F.h"
#include "TGraph2D.h"
#include "TGraph2DErrors.h"
#include "TSystem.h"
#include "Buttons.h"

#include <iostream>
#include <cmath>

#include "CDataHime.hh"
#include "CDataHimeSim.hh"
#include "CKinema.hh"
#include "CParticle.hh"

void hime2();
void SimToExp(CDataHimeSim* data, CDataHime* himedata);
int Analysis(CDataHime* himedata, TGraph2DErrors* graph);
void EventLoop();
void line(double t, double *p, double &x, double &y, double &z);
double distance2(double x,double y,double z, double ex, double ey, double ez, double *p);
void SumDistance2(int &, double *, double & sum, double * par, int );
void line3Dfit(TGraph2DErrors* gr, double parFit[4]);
void DrawLine(double parFit[4], Color_t color);

//#define VISUALIZE 1

static const Double_t thr(3.0);
static const Double_t treso = 0.13;
static const Double_t length = 100.;
static const Double_t efflength = 36.;
static const Double_t width = 4.;
static const Double_t depth = 2.;
static const Double_t vl = 15.;
static const std::deque<CDataHime::OneData>::size_type multihit = 3;

void hime2() 
{
  TCanvas* canvas = new TCanvas("canvas","",200,10,700,700);

   //   TFile *file = TFile::Open("root/proton.root","READ"); if(!file) return;
   //   TFile *file = TFile::Open("root/neutron.root","READ"); if(!file) return;
   //   TFile *file = TFile::Open("root/neutron_pen.root","READ"); if(!file) return;
  //   TFile *file = TFile::Open("root/neutron_pen3.root","READ"); if(!file) return;
   TFile *file = TFile::Open("root/neutron_pen4.root","READ"); if(!file) return;
   if(!file) return;
   TTree *tree; file->GetObject("Koyomi",tree);
   tree->Print();

   CDataHimeSim* data = 0;
   TBranch *bdata = 0;
   if(tree->SetBranchAddress("hime",&data,&bdata) == TTree::kMissingBranch) return;

   TH1D* hpa = new TH1D("hpa", "title", 100, 731.-10, 731.+10);
   TH1D* hpx = new TH1D("hpx", "title", 100, 0.-10, 0.+10);
   TH1D* hpy = new TH1D("hpy", "title", 100, 0.-10, 0.+10);
   TH1D* hpz = new TH1D("hpz", "title", 100, 731.-10, 731.+10);

   Long64_t nentries = tree->GetEntriesFast();
   //   nentries = 1000;
   for(Long64_t i=0; i<nentries; i++) {
     Long64_t entry = tree->LoadTree(i);
     bdata->GetEntry(entry); if(data->fDequeOfOneData.empty()) continue;

     CDataHime* himedata = new CDataHime;
     SimToExp(data, himedata);

     TGraph2DErrors* graph = new TGraph2DErrors;
     if(Analysis(himedata, graph) != 0) continue;

     hpa->Fill(himedata->fP.P());
     hpx->Fill(himedata->fP.Px());
     hpy->Fill(himedata->fP.Py());
     hpz->Fill(himedata->fP.Pz());

     if(0){
       std::cout << himedata->fX.X() << " "
		 << himedata->fX.Y() << " "
		 << himedata->fX.Z() << " "
	 	 << himedata->fX.T() << " "
		 << himedata->fP.P() << " "       
		 << himedata->fP.E() << " "
		 << himedata->fP.E()-himedata->fP.M() << " "
		 << std::endl;
     }
   }

   canvas->Divide(2,2);
   canvas->cd(1);
   hpa->Draw();
   hpa->Fit("gaus");
   canvas->cd(2);
   hpx->Draw();
   hpx->Fit("gaus");
   canvas->cd(3);
   hpy->Draw();
   hpy->Fit("gaus");
   canvas->cd(4);
   hpz->Draw();
   hpz->Fit("gaus");

   tree->ResetBranchAddresses();
}

void SimToExp(CDataHimeSim* simdata, CDataHime* himedata)
{
  std::deque<CDataHimeSim::OneData>::iterator itsim;
  itsim = simdata->fDequeOfOneData.begin();
  while(itsim != simdata->fDequeOfOneData.end()){
    std::vector<Int_t> id;
    id.push_back(itsim->fIdMother);
    id.push_back(itsim->fId);

    Double_t xl = length/2.-itsim->fPosition.X();
    Double_t xr = itsim->fPosition.X()+length/2.;
    Double_t yl = length/2.-itsim->fPosition.Y();
    Double_t yr = itsim->fPosition.Y()+length/2.;

    Double_t tl, tr, al, ar;
    if(id[0]%2){
      tl = itsim->fTime + yl/vl;
      tr = itsim->fTime + yr/vl;
      tl += gRandom->Gaus(0,treso);  
      tr += gRandom->Gaus(0,treso);  
      al = itsim->fEdep*exp(-yl/length);
      ar = itsim->fEdep*exp(-yr/length);
    }else{
      tl = itsim->fTime + xl/vl;
      tr = itsim->fTime + xr/vl;
      tl += gRandom->Gaus(0,treso);  
      tr += gRandom->Gaus(0,treso);  
      al = itsim->fEdep*exp(-xl/length);
      ar = itsim->fEdep*exp(-xr/length);
    }

    CDataHime::OneData* onedata = himedata->Find(id[0], id[1]);
    if(onedata){
      if(onedata->fTL > tl) onedata->fTL = tl;
      if(onedata->fTR > tr) onedata->fTR = tr;
      onedata->fAL += al;
      onedata->fAR += ar;
    }else{
      himedata->fDequeOfOneData.push_back(CDataHime::OneData(id, tl, tr, al, ar));
    }

    ++itsim;
  }

  std::deque<CDataHime::OneData>::iterator it;
  it = himedata->fDequeOfOneData.begin();
  while(it != himedata->fDequeOfOneData.end()){
    it->fT = (it->fTL+it->fTR-length/vl)/2;
    it->fE = sqrt(it->fAL*it->fAR);

    Double_t x = efflength/2.-it->fId[1]*width - width/2.;
    //    Double_t x = efflength/2.-it->fId[1]*width - width/2. + gRandom->Uniform(-width/2., width/2.);
    Double_t y = vl*(it->fTL-it->fTR)/2.;
    Double_t z = it->fId[0]*2.*depth+701;
    //    Double_t z = it->fId[0]*2.*depth+701 + gRandom->Uniform(-depth/2., depth/2.);
    //    Double_t z = it->fId[0]*width+700;
    if(it->fId[0]%2){
      it->fX.SetX(x);
      it->fX.SetY(y);
    }else{
      it->fX.SetX(y);
      it->fX.SetY(x);
    }
    it->fX.SetZ(z);
    ++it;
  }
}

int Analysis(CDataHime* himedata, TGraph2DErrors* graph)
{
  std::deque<CDataHime::OneData>::iterator it;
  it = himedata->fDequeOfOneData.begin();
  while(it != himedata->fDequeOfOneData.end()){
    if(it->fE < thr){
      it = himedata->fDequeOfOneData.erase(it);
      return 1;
    }
    ++it;
  }

  if(himedata->fDequeOfOneData.size() < multihit) return 1;

  Int_t point = 0;
  it = himedata->fDequeOfOneData.begin();
  while(it != himedata->fDequeOfOneData.end()){
    graph->SetPoint(point, it->fX.X(), it->fX.Y(), it->fX.Z());
    if(it->fId[0]%2){
      graph->SetPointError(point, width/sqrt(12), vl*treso/sqrt(2), depth/sqrt(12));
    }else{	 
      graph->SetPointError(point, vl*treso/sqrt(2), width/sqrt(12), depth/sqrt(12));
    }

    ++point;
    ++it;
  }

  double parFit[4];
  line3Dfit(graph, parFit);

  std::deque<CDataHime::OneData>::iterator itmin;
  itmin = himedata->fDequeOfOneData.begin();
  it = himedata->fDequeOfOneData.begin();
  while(it != himedata->fDequeOfOneData.end()){
    if(itmin->fX.Z() > it->fX.Z()) itmin = it;
    ++it;
  }

  //analysis of timing
  double tmin = 0;
  int count = 0;
  it = himedata->fDequeOfOneData.begin();
  while(it != himedata->fDequeOfOneData.end()){
    //先頭でhitしてから4層通過分の時間程度以降は切ってしまう
    //energy lossを考慮できるようになったら条件を緩められると思われる。
    if(it->fT - itmin->fT < 0.6){
      double distance = (it->fX - itmin->fX).Mag();
      TVector3 xs = itmin->fX;
      double t1 = 700., x1, y1, z1;
      line(t1,parFit,x1,y1,z1);
      double t2 = 720., x2, y2, z2;
      line(t2,parFit,x2,y2,z2);
      TVector3 xf(x2-x1, y2-y1, z2-z1);
      double theta = xf.Angle(xs);

      double beta;

      // non-relativistic
//      {
//	double ek = 250.*cos(theta)*cos(theta);
//	beta = sqrt(ek*ek + 2.*ek*939.)/(ek+939.);
//      }

      // relativistic
      {
	double mu = 931.494;
	double mp = 7.289 + mu;
	double mn = 8.071 + mu;

	CParticle pin(mp), nin(mn), pout(mp), nout(mn);
	pin.k(250);
	nin.k(0);

	static CKinema kinema;
	kinema.Scatter(theta, 0., pin, nin, nout, pout);
	beta = nout.beta();
      }

      tmin += it->fT - distance/beta/29.979;
      //      std::cout << theta << " " << nout.e() << " " << beta << " "  << it->fT << " " << distance << " " << distance/beta/29.979 << " " << itmin->fT << std::endl;
      ++count;
    }
    ++it;
  }

  tmin /= count;

  Double_t x,y,z;
  line(itmin->fX.Z(),parFit,x,y,z);

  if(1){
    himedata->fX.SetX(x);
    himedata->fX.SetY(y);
    himedata->fX.SetZ(z);
  }else{
    himedata->fX.SetX(itmin->fX.X());
    himedata->fX.SetY(itmin->fX.Y());
    himedata->fX.SetZ(itmin->fX.Z());
  }

  himedata->fX.SetT(tmin);
  //  himedata->fX.SetT(itmin->fT);

  himedata->CalMomentum();

#ifdef VISUALIZE
  graph->SetMarkerStyle(8);
  graph->SetMarkerSize(1);
  graph->GetXaxis()->SetLimits(-50., 50.);
  graph->GetYaxis()->SetLimits(-50., 50.);
  graph->SetMinimum(700.);
  graph->SetMaximum(720.);

  if(1){
    graph->Draw("PERR");
  }else{
    static bool init = true;
    if(init){
      graph->Draw("PERR");
      init = false;
    }else{
      graph->Draw("PERRSAME");
    }
  }

  DrawLine(parFit, kRed);

  TCanvas* canvas = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("canvas");
  canvas->Update();

  EventLoop();
#endif

  return 0;
}

void EventLoop()
{
  TCanvas* canvas = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("canvas");

  canvas->HandleInput(kNoEvent, 0, 0);
  while (!gSystem->ProcessEvents() && canvas) {
    canvas = (TCanvas*)gROOT->GetSelectedPad();
    Int_t event = canvas->GetEvent();
    if(event == kKeyPress) break;
    if(gSystem->ProcessEvents()) break;
    gSystem->Sleep(100);
  }
}

void line(double t, double *p, double &x, double &y, double &z)
{ 
  x = p[0] + p[1]*t; 
  y = p[2] + p[3]*t;
  z = t; 
} 

// double distance2(double x,double y,double z, double ex, double ey, double ez, double *p)
// { 
//   ROOT::Math::XYZVector xp(x,y,z); 
//   ROOT::Math::XYZVector x0(p[0], p[2], 0. ); 
//   ROOT::Math::XYZVector x1(p[0] + p[1], p[2] + p[3], 1. ); 
//   ROOT::Math::XYZVector u = (x1-x0).Unit(); 
//   double d2 = ((xp-x0).Cross(u)) .Mag2(); 
//   return d2; 
// }

double distance2(double x,double y,double z, double ex, double ey, double /*ez*/, double *p)
{ 
  double t = z;
  double xt, yt, zt;
  line(t, p, xt, yt, zt);
  double d2 = pow((xt-x)/ex, 2.) + pow((yt-y)/ey, 2.);
  return d2; 
}

void SumDistance2(int &, double *, double & sum, double * par, int )
{ 
  static bool first = true; 

  TGraph2DErrors * gr = dynamic_cast<TGraph2DErrors*>( (TVirtualFitter::GetFitter())->GetObjectFit() );
  assert(gr != 0);
  double* x = gr->GetX();
  double* y = gr->GetY();
  double* z = gr->GetZ();
  double* ex = gr->GetEX();
  double* ey = gr->GetEY();
  double* ez = gr->GetEZ();
  int npoints = gr->GetN();
  sum = 0;
  for (int i  = 0; i < npoints; ++i) { 
    double d = distance2(x[i],y[i],z[i], ex[i], ey[i], ez[i], par); 
    sum += d;
#ifdef DEBUG
    if (first) std::cout << "point " << i << "\t" 
			 << x[i] << "\t" 
			 << y[i] << "\t" 
			 << z[i] << "\t" 
			 << std::sqrt(d) << std::endl; 
#endif
  }
  if(first) std::cout << "Total sum2 = " << sum << std::endl;

  first = false;
}

void line3Dfit(TGraph2DErrors* gr, double parFit[4])
{
  TVirtualFitter *min = TVirtualFitter::Fitter(0,4);
  min->SetObjectFit(gr);
  min->SetFCN(SumDistance2);
   
  Double_t arglist[10];
  arglist[0] = -1;
  min->ExecuteCommand("SET PRINT",arglist,1);
  
  double pStart[4] = {1,1,1,1};
  min->SetParameter(0,"x0",pStart[0],0.01,0,0);
  min->SetParameter(1,"Ax",pStart[1],0.01,0,0);
  min->SetParameter(2,"y0",pStart[2],0.01,0,0);
  min->SetParameter(3,"Ay",pStart[3],0.01,0,0);
    
  arglist[0] = 1000; // number of function calls 
  arglist[1] = 0.001; // tolerance 
  min->ExecuteCommand("MIGRAD",arglist,2);

  //if (minos) min->ExecuteCommand("MINOS",arglist,0);
  int nvpar,nparx; 
  double amin,edm, errdef;
  min->GetStats(amin,edm,errdef,nvpar,nparx);
  //  min->PrintResults(1,amin);
  //  gr->Draw("p0");

  // get fit parameters
  for (int i = 0; i <4; ++i) parFit[i] = min->GetParameter(i);
}

void DrawLine(double parFit[4], Color_t color)
{
  int n = 1000;
  double t0 = 700;
  double dt = 720;
  //  double t0 = -50;
  //  double dt = 100;
  TPolyLine3D *l = new TPolyLine3D(n);
  for (int i = 0; i <n;++i) {
    double t = t0+ dt*i/n;
    double x,y,z;
    line(t,parFit,x,y,z);
    l->SetPoint(i,x,y,z);
  }

#ifdef VISUALIZE
  l->SetLineColor(color);
  l->Draw("same");
#endif
}
