   
#include <TMath.h>
#include <TGraph2D.h>
#include <TRandom2.h>
#include <TStyle.h>
#include <TCanvas.h>
#include <TF2.h>
#include <TH1.h>
#include <TVirtualFitter.h>
#include <TPolyLine3D.h>
#include <Math/Vector3D.h>

#include "TROOT.h"
#include "TFile.h"
#include "TTree.h"
#include "TCanvas.h"
#include "TH2D.h"
#include "TH1F.h"
#include "TH3F.h"
#include "TGraph2D.h"
#include "TGraph2DErrors.h"
#include "TSystem.h"
#include "Buttons.h"

#include <iostream>
#include <deque>
#include <cmath>

#include "CDataHime.hh"
#include "CDataHimeSim.hh"
#include "CKinema.hh"
#include "CParticle.hh"

//#define VISUALIZE 1

void hime_track();
void SimToExp(CDataHimeSim* data, CDataHime* expdata);
void Analysis(CDataHime* expdata);

void EventLoop();
void line(double t, double *p, double &x, double &y, double &z);
double distance2(double x,double y,double z, double ex, double ey, double ez, double *p);
void SumDistance2(int &, double *, double & sum, double * par, int );
double line3Dfit(TGraph2DErrors* gr, double parFit[4]);
void DrawLine(double parFit[4], Color_t color);

static const Double_t light_v = 29.979;

static const Double_t thr = 3.0;
//static const Double_t treso = 0.13;
static const Double_t treso = 0.20;
static const Double_t length = 100.;
static const Double_t efflength = 40.;
static const Double_t width = 4.;
static const Double_t depth = 2.;
static const Double_t vl = 15.;
static const Double_t aminmin = 20.;
//static const Double_t nenergy = 250 - 15;
static const Double_t nenergy = 192 - 15;

//static const std::deque<CDataHime::OneData>::size_type multihit = 2;
static const std::deque<CDataHime::OneData>::size_type multihit = 3;
//static const std::deque<CDataHime::OneData>::size_type multihit = 4;
//static const std::deque<CDataHime::OneData>::size_type multihit = 5;

void hime_track() 
{
  TCanvas* canvas = new TCanvas("canvas","",200,10,700,700);
  std::deque<TVector3> momentum;

  {
    //    TFile* file = TFile::Open("root/neutron_pen8.root","READ");
    TFile* file = TFile::Open("root/11Li.root","READ");
    if(!file) return;

    TTree* tree;
    file->GetObject("Koyomi",tree);
    tree->Print();

    TFile* workfile = TFile::Open("root/temp.root","recreate");
    if(!workfile) return;

    TTree* newtree = new TTree("Hitagi", "title");
    new TH1D("hamin", "title", 100, 0., 20.);

    CDataHimeSim* data = 0;
    TBranch *bdata = 0;
    if(tree->SetBranchAddress("hime",&data,&bdata) == TTree::kMissingBranch) return;

    CDataHime* expdata = new CDataHime;
    //     newtree->Branch("hime", &expdata, 32000, 0);
    newtree->Branch("hime", &expdata);

    Long64_t nentries = tree->GetEntriesFast();
    //    nentries = 100000;
    for(Long64_t i=0; i<nentries; i++) {
      std::cout << "\r event:" << i << std::flush;

      expdata->Reset();

      Long64_t entry = tree->LoadTree(i);
      bdata->GetEntry(entry);

      SimToExp(data, expdata);
      Analysis(expdata);

      if(!expdata->fEventFlag) continue;

      momentum.push_back(data->fMomentum);
      newtree->Fill();
    }
    std::cout << std::endl;

    tree->ResetBranchAddresses();
    
    workfile->Write();
    workfile->Close();
    file->Close();
  }

  {
    TFile* file = TFile::Open("root/temp.root","READ");
    if(!file) return;

    TTree* tree;
    file->GetObject("Hitagi",tree);
    tree->Print();

    CDataHime* data = 0;
    TBranch *bdata = 0;
    if(tree->SetBranchAddress("hime",&data,&bdata) == TTree::kMissingBranch) return;

    TH1D* hpa = new TH1D("hpa", "title", 100, 0.-10, 0.+10);
    TH1D* hpx = new TH1D("hpx", "title", 100, 0.-10, 0.+10);
    TH1D* hpy = new TH1D("hpy", "title", 100, 0.-10, 0.+10);
    TH1D* hpz = new TH1D("hpz", "title", 100, 0.-10, 0.+10);

    Long64_t nentries = tree->GetEntriesFast();
    std::deque<TVector3>::iterator it = momentum.begin();
    for(Long64_t i=0; i<nentries; i++) {
      Long64_t entry = tree->LoadTree(i);
      bdata->GetEntry(entry);

      if(!data->fEventFlag){
	++it;
	continue;
      }

      hpa->Fill(data->fP.P()-it->Mag());
      hpx->Fill(data->fP.Px()+it->Px());
      hpy->Fill(data->fP.Py()+it->Py());
      hpz->Fill(data->fP.Pz()-it->Pz());
      ++it;
    }

    canvas->Divide(2,2);
    canvas->cd(1);
//    hpa->Draw();
//    hpa->Fit("gaus");
    TH1D* hamin;
    gDirectory->GetObject("hamin", hamin);
    hamin->Draw();
    canvas->cd(2);
    hpx->Draw();
    hpx->Fit("gaus");
    canvas->cd(3);
    hpy->Draw();
    hpy->Fit("gaus");
    canvas->cd(4);
    hpz->Draw();
    hpz->Fit("gaus");
    canvas->Update();

    tree->ResetBranchAddresses();
  }
}

void SimToExp(CDataHimeSim* simdata, CDataHime* expdata)
{
  std::deque<CDataHimeSim::OneData>::iterator itsim;
  itsim = simdata->fDequeOfOneData.begin();
  while(itsim != simdata->fDequeOfOneData.end()){
    std::vector<Int_t> id;
    id.push_back(itsim->fIdMother);
    id.push_back(itsim->fId);

    Double_t xl = length/2.-itsim->fPosition.X();
    Double_t xr = itsim->fPosition.X()+length/2.;
    Double_t yl = length/2.-itsim->fPosition.Y();
    Double_t yr = itsim->fPosition.Y()+length/2.;

    Double_t tl, tr, al, ar;
    if(id[0]%2){
      tl = itsim->fTime + yl/vl;
      tr = itsim->fTime + yr/vl;
      al = itsim->fEdep*exp(-yl/length);
      ar = itsim->fEdep*exp(-yr/length);
    }else{
      tl = itsim->fTime + xl/vl;
      tr = itsim->fTime + xr/vl;
      al = itsim->fEdep*exp(-xl/length);
      ar = itsim->fEdep*exp(-xr/length);
    }

    CDataHime::OneData* onedata = expdata->Find(id[0], id[1]);
    if(onedata){
      if(onedata->fTL > tl) onedata->fTL = tl;
      if(onedata->fTR > tr) onedata->fTR = tr;
      onedata->fAL += al;
      onedata->fAR += ar;
    }else{
      expdata->fDequeOfOneData.push_back(CDataHime::OneData(id, tl, tr, al, ar));
    }

    ++itsim;
  }

  std::deque<CDataHime::OneData>::iterator it;
  it = expdata->fDequeOfOneData.begin();
  while(it != expdata->fDequeOfOneData.end()){
    it->fTL += gRandom->Gaus(0,treso);  
    it->fTR += gRandom->Gaus(0,treso);  

    it->fT = (it->fTL+it->fTR-length/vl)/2;
    it->fE = sqrt(it->fAL*it->fAR);

    Double_t x = efflength/2.-it->fId[1]*width - width/2.;
    //    Double_t x = efflength/2.-it->fId[1]*width - width/2. + gRandom->Uniform(-width/2., width/2.);
    Double_t y = vl*(it->fTL-it->fTR)/2.;
    Double_t z = it->fId[0]*2.*depth+701;
    //    Double_t z = it->fId[0]*2.*depth+701 + gRandom->Uniform(-depth/2., depth/2.);
    if(it->fId[0]%2){
      it->fX.SetX(x);
      it->fX.SetY(y);
    }else{
      it->fX.SetX(y);
      it->fX.SetY(x);
    }
    it->fX.SetZ(z);
    ++it;
  }
}

void Analysis(CDataHime* expdata)
{
  TGraph2DErrors* graph = new TGraph2DErrors;

  std::deque<CDataHime::OneData>::iterator it;
  it = expdata->fDequeOfOneData.begin();
  while(it != expdata->fDequeOfOneData.end()){
    if(it->fE < thr){
      it = expdata->fDequeOfOneData.erase(it);
    }else{
      ++it;
    }
  }

  if(expdata->fDequeOfOneData.size() < multihit){
    expdata->fEventFlag = 0;
    return;
  }

  Int_t point = 0;
  it = expdata->fDequeOfOneData.begin();
  while(it != expdata->fDequeOfOneData.end()){
    graph->SetPoint(point, it->fX.X(), it->fX.Y(), it->fX.Z());
    if(it->fId[0]%2){
      graph->SetPointError(point, width/sqrt(12), vl*treso/sqrt(2), depth/sqrt(12));
    }else{	 
      graph->SetPointError(point, vl*treso/sqrt(2), width/sqrt(12), depth/sqrt(12));
    }

    ++point;
    ++it;
  }

  double parFit[4];
  double amin = line3Dfit(graph, parFit);
  TH1D* hamin;
  gDirectory->GetObject("hamin", hamin);
  hamin->Fill(amin);

  //ここがものすごく効く->つまり変なのが混じりすぎ->MeVee変換必須
  //もしくは選択的にトラックを引くアルゴリズムが必要。
  //たとえばamin<5になるまで、計算に入れる点を変えるとか。
  if(amin > aminmin){
  //  if(amin < 6. || amin > 8.){
    expdata->fEventFlag = 0;
    return;
  }

  std::deque<CDataHime::OneData>::iterator itmin;
  itmin = expdata->fDequeOfOneData.begin();
  it = expdata->fDequeOfOneData.begin();
  while(it != expdata->fDequeOfOneData.end()){
    if(itmin->fX.Z() > it->fX.Z()) itmin = it;
    ++it;
  }

  //analysis of timing
  double t1 = 700., x1, y1, z1;
  line(t1,parFit,x1,y1,z1);
  double t2 = 720., x2, y2, z2;
  line(t2,parFit,x2,y2,z2);
  TVector3 xf(x2-x1, y2-y1, z2-z1);
  TVector3 xs = itmin->fX;
  double theta = xf.Angle(xs);

// derive recoile proton energy from theta(lab)
// non-relativistic
//	double ek = 250.*cos(theta)*cos(theta);
//	beta = sqrt(ek*ek + 2.*ek*939.)/(ek+939.);

  double mu = 931.494;
  double mp = 7.289 + mu;
  double mn = 8.071 + mu;

  CParticle pin(mp), nin(mn), pout(mp), nout(mn);
  pin.k(nenergy);
  nin.k(0);

  static CKinema kinema;
  kinema.Scatter(theta, 0., pin, nin, nout, pout);

  double tmin = 0;
  int count = 0;
  it = expdata->fDequeOfOneData.begin();
  while(it != expdata->fDequeOfOneData.end()){
    //ここの処理を整理・見直すべき
    double tsum = -1.;
    //先頭でhitしてから4層通過分程度の時間以降は切ってしまう
    //energy lossを考慮できるようになったら条件を緩められると思われる。
    if(it == itmin){
      tsum = 0;
    }else if(it->fT - itmin->fT < 0.8){
      double xmin, ymin, zmin;
      line(it->fX.Z(), parFit, xmin, ymin, zmin);
      TVector3 Xmin(xmin, ymin, zmin);      
      double distance = (Xmin - itmin->fX).Mag();

      CParticle ntemp = nout;
      tsum  = CKinema::BetheBloch::TimeOut
	("p", "Scint", ntemp, distance/2., 300.);//plasticは2cmしかない
      if(tsum >= 0.){
	tsum += (distance/2.)/((nout.beta()+ntemp.beta())*29.979/2.);//ほんとは2cm+2cm+2cm+....と交互に時間を足していかなければならない	
      }
    }
    if(tsum >= 0.){
      if(abs((it->fT - tsum) - itmin->fT) < treso*2){
	tmin += it->fT - tsum;
	++count;
      }
    }
    ++it;
  }

  tmin /= count;

  Double_t x,y,z;
  line(itmin->fX.Z(),parFit,x,y,z);

  expdata->fX.SetX(x);
  expdata->fX.SetY(y);
  expdata->fX.SetZ(z);
  expdata->fX.SetT(tmin);

  expdata->CalMomentum();

#ifdef VISUALIZE
  graph->SetMarkerStyle(8);
  graph->SetMarkerSize(1);
  graph->GetXaxis()->SetLimits(-length/2., length/2.);
  graph->GetYaxis()->SetLimits(-length/2., length/2.);
  graph->SetMinimum(700.);
  graph->SetMaximum(720.);

  if(1){
    graph->Draw("PERR");
  }else{
    static bool init = true;
    if(init){
      graph->Draw("PERR");
      init = false;
    }else{
      graph->Draw("PERRSAME");
    }
  }

  DrawLine(parFit, kRed);

  TCanvas* canvas = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("canvas");
  canvas->Update();

  EventLoop();
#endif

  expdata->fEventFlag = 1;

  return;
}

void EventLoop()
{
  TCanvas* canvas = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("canvas");

  canvas->HandleInput(kNoEvent, 0, 0);
  while (!gSystem->ProcessEvents() && canvas) {
    canvas = (TCanvas*)gROOT->GetSelectedPad();
    Int_t event = canvas->GetEvent();
    if(event == kKeyPress) break;
    if(gSystem->ProcessEvents()) break;
    gSystem->Sleep(100);
  }
}

void line(double t, double *p, double &x, double &y, double &z)
{ 
  x = p[0] + p[1]*t; 
  y = p[2] + p[3]*t;
  z = t; 
} 

// double distance2(double x,double y,double z, double ex, double ey, double ez, double *p)
// { 
//   ROOT::Math::XYZVector xp(x,y,z); 
//   ROOT::Math::XYZVector x0(p[0], p[2], 0. ); 
//   ROOT::Math::XYZVector x1(p[0] + p[1], p[2] + p[3], 1. ); 
//   ROOT::Math::XYZVector u = (x1-x0).Unit(); 
//   double d2 = ((xp-x0).Cross(u)) .Mag2(); 
//   return d2; 
// }

double distance2(double x,double y,double z, double ex, double ey, double /*ez*/, double *p)
{ 
  double t = z;
  double xt, yt, zt;
  line(t, p, xt, yt, zt);
  double d2 = pow((xt-x)/ex, 2.) + pow((yt-y)/ey, 2.);
  return d2; 
}

void SumDistance2(int &, double *, double & sum, double * par, int )
{ 
  static bool first = true; 

  TGraph2DErrors * gr = dynamic_cast<TGraph2DErrors*>( (TVirtualFitter::GetFitter())->GetObjectFit() );
  assert(gr != 0);
  double* x = gr->GetX();
  double* y = gr->GetY();
  double* z = gr->GetZ();
  double* ex = gr->GetEX();
  double* ey = gr->GetEY();
  double* ez = gr->GetEZ();
  int npoints = gr->GetN();
  sum = 0;
  for (int i  = 0; i < npoints; ++i) { 
    double d = distance2(x[i],y[i],z[i], ex[i], ey[i], ez[i], par); 
    sum += d;
  }
  if(first) std::cout << "Total sum2 = " << sum << std::endl;

  first = false;
}

double line3Dfit(TGraph2DErrors* gr, double parFit[4])
{
  TVirtualFitter *min = TVirtualFitter::Fitter(0,4);
  min->SetObjectFit(gr);
  min->SetFCN(SumDistance2);
   
  Double_t arglist[10];
  arglist[0] = -1;
  min->ExecuteCommand("SET PRINT",arglist,1);
  
  double pStart[4] = {1,1,1,1};
  min->SetParameter(0,"x0",pStart[0],0.01,0,0);
  min->SetParameter(1,"Ax",pStart[1],0.01,0,0);
  min->SetParameter(2,"y0",pStart[2],0.01,0,0);
  min->SetParameter(3,"Ay",pStart[3],0.01,0,0);
    
  arglist[0] = 1000; // number of function calls 
  arglist[1] = 0.001; // tolerance 
  min->ExecuteCommand("MIGRAD",arglist,2);

  //if (minos) min->ExecuteCommand("MINOS",arglist,0);
  int nvpar,nparx; 
  double amin,edm, errdef;
  min->GetStats(amin,edm,errdef,nvpar,nparx);

  //  min->PrintResults(1,amin);
  //  gr->Draw("p0");

  // get fit parameters
  for (int i = 0; i <4; ++i) parFit[i] = min->GetParameter(i);

  return amin;
}

void DrawLine(double parFit[4], Color_t color)
{
  int n = 1000;
  double t0 = 700;
  double dt = 720;
  //  double t0 = -50;
  //  double dt = 100;
  TPolyLine3D *l = new TPolyLine3D(n);
  for (int i = 0; i <n;++i) {
    double t = t0+ dt*i/n;
    double x,y,z;
    line(t,parFit,x,y,z);
    l->SetPoint(i,x,y,z);
  }

#ifdef VISUALIZE
  l->SetLineColor(color);
  l->Draw("same");
#endif
}
