#include <iostream>
#include <iomanip>
#include <sstream>
#include <deque>
#include <cstdlib>

#include "TRandom3.h" 
#include "TObject.h"

#include "NMadoka.hh"
#include "CDataHimeSim.hh"

void OneRun(Int_t runnumber);
void OneEvent(Int_t eventnumber);

static const Int_t RUN = 10;
static const Int_t EVENT = 10000;
static const Int_t INNEREVENT = 10;

int example_madoka()
{
  for(Int_t runnumber = 0; runnumber < RUN; ++runnumber){
    OneRun(runnumber);
  }

  return 0;
}

void OneRun(Int_t runnumber)
{
  std::cout << "run:" << runnumber << std::endl;

  std::ostringstream oss;
  oss << "root/madoka/run" << std::setw(4) << std::setfill('0') << runnumber << ".root";
  NMadoka::OpenFile(oss.str(), "recreate");
  NMadoka::CreateTree("Koyomi", "Koyomi is Hero of Bakemono-Gatari");
  CDataHimeSim* data = new CDataHimeSim("Nadeko");
  //  NMadoka::CreateBranch("Koyomi", "hime", data);
  NMadoka::CreateBranch("Koyomi", "hime", data, 32000, 0);

  for(Int_t i = 0; i < EVENT; ++i) OneEvent(i);

  std::cout << std::endl;

  NMadoka::Write();
  NMadoka::CloseFile();
}

void OneEvent(Int_t eventnumber)
{
  std::cout << "\r" << "event:" << eventnumber << std::flush;

  TRandom3* random = new TRandom3;
  random->SetSeed();

  double x,y,z;
  double r = random->Rndm();
  random->Sphere(x,y,z,r);

  //fast but danger
#if 0
  CDataHimeSim* data = (CDataHimeSim*)NMadoka::FindObject("Nadeko");
#endif

  //slow but safe
#if 1
  CDataHimeSim* data = 0;
  TObject* obj = NMadoka::FindObject("Nadeko");
  if(obj && obj->InheritsFrom(CDataHimeSim::Class())){
    data = (CDataHimeSim*)obj;
  }else{
    std::abort();
  }
#endif

  data->Reset();

  data->fEventFlag = random->Integer(2);
  for(UInt_t j = 0; j < 1+random->Integer(INNEREVENT); ++j){
    CDataHimeSim::OneData onedata(random->Integer(2), random->Integer(3), "name", x, y);
    data->fDequeOfOneData.push_back(onedata);
  }

  NMadoka::Fill("Koyomi");

  delete random;
}
