#ifndef CBRANCHMANAGER_HH
#define CBRANCHMANAGER_HH

#include "TCut.h"
#include "TChain.h"

#include <string>

#include "IArrayManager.hh"

class TTree;
class TBranch;
class TLeaf;
class CTreeManager;
class CCreateHist;

namespace NAlias
{
  extern CTreeManager* treemanager;
}

class CBranchManager : public IArrayManager
{
  friend class CCreateHist;
  friend class CRead;

public:
  CBranchManager() : IArrayManager()
  {
    IArrayManager::listname = "branchmanager";
  }
  ~CBranchManager(){;}

  TBranch* GetBranch(int id);
  TBranch* GetBranch(std::string name);
  TBranch* GetBranch();
  TLeaf* GetLeaf(int id);
  TLeaf* GetLeaf(std::string name);
  TLeaf* GetLeaf();
  void Push(TTree* t);
  void Draw(std::string name, const TCut* selection = 0);
  void Draw(int id);
  void Draw(int idx, int idy, int idz = -1, const TCut* selection = 0);
  void Draw(int idx, int idy, const TCut* selection = 0);
  void Draw(int idx, const TCut* selection);

  template <class T>
  bool SetValueAddress(std::string name, T value);
  template <class T>
  bool SetValueAddress(int id, T value);

private:
  std::string FunctionName(std::string functionname) const;
};

#include "CTreeManager.hh"

template <class T>
inline bool CBranchManager::SetValueAddress(std::string name, T value)
{
  if(!ExistName(name)) return false;

  TTree* tree = NAlias::treemanager->GetTree(GetTitle(name));
  if(!Check(tree)) return false;
  TBranch* tempbranch;
  tree->SetBranchAddress(name.c_str(), value, &tempbranch);

  return true;
}

template <class T>
inline bool CBranchManager::SetValueAddress(int id, T value)
{
  if(!ExistId(id)) return false;

  SetValueAddress(GetName(id), value);
  return true;
}

#endif
