#include "TROOT.h" 
#include "TTree.h"
#include "TChain.h"
#include "TObjArray.h"
#include "TBranch.h"
#include "TLeaf.h"
#include "TCut.h"

#include <iostream>
#include <string>
#include <sstream>

#include "CBranchManager.hh"
#include "NGlobal.hh"

TBranch* CBranchManager::GetBranch(std::string name)
{
  TObject* obj = gROOT->FindObjectAny(name.c_str());
  if(obj && obj->InheritsFrom("TBranch")) return (TBranch*)obj;

  std::ostringstream os;
  os << FunctionName("GetBranch") << name << " dosn't exist as TBranch";
  NGlobal::Message(os.str(), kMID);

  return 0;
}

TBranch* CBranchManager::GetBranch(int id)
{
  std::string name = GetName(id);
  if(!Check(name)) return 0;

  return GetBranch(name);
}

TBranch* CBranchManager::GetBranch()
{
  std::string name = GetName();  
  if(!Check(name)) return 0;

  return GetBranch(name);
}

TLeaf* CBranchManager::GetLeaf(std::string name)
{
  TObject* obj = gROOT->FindObjectAny(name.c_str());
  if(obj && obj->InheritsFrom("TLeaf")) return (TLeaf*)obj;

  std::ostringstream os;
  os << FunctionName("GetLeaf") << name << " dosn't exist as TLeaf";
  NGlobal::Message(os.str(), kMID);

  return 0;
}

TLeaf* CBranchManager::GetLeaf(int id)
{
  std::string name = GetName(id);
  if(!Check(name)) return 0;

  return GetLeaf(name);
}

TLeaf* CBranchManager::GetLeaf()
{
  std::string name = GetName();  
  if(!Check(name)) return 0;

  return GetLeaf(name);
}

void CBranchManager::Push(TTree* tree)
{
  if(!tree){
    std::cout << "[CBranchManager::Push]$ not valid pointer" << std::endl;
    return;
  }

  TObjArray* fBranches = tree->GetListOfBranches();
  int branch_entries = fBranches->GetEntriesFast();

  for(int i(0);i < branch_entries; ++i){
    TBranch* branch= (TBranch*)(*fBranches)[i];
    if(!branch) continue;
    TObjArray* fLeaves = branch->GetListOfLeaves();
    int leaf_entries = fLeaves->GetEntriesFast();
    //insert leaves to list
    if(leaf_entries > 1){
      for(int j(0);j < leaf_entries; ++j){
	TLeaf* leaf= (TLeaf*)(*fLeaves)[j];
	if(!leaf) continue;
	std::ostringstream os;
	os << branch->GetName() << "." << leaf->GetName();
	IArrayManager::Push(os.str().c_str(), tree->GetName());
      }
    }else{
      IArrayManager::Push(branch->GetName(), tree->GetName());
    }
  }    

  std::cout << "[CBranchManager::Push]$ read in "
    	    << tree->GetEntriesFast() << " entries"
    //	    << tree->GetEntries() << " entries"
	    << std::endl;
}

void CBranchManager::Draw(std::string name, const TCut* selection)
{
  TTree* tree = NAlias::treemanager->GetTreeTop();
  if(!Check(tree)) return;

  if(selection){
    tree->Draw(name.c_str(), *selection);
  }else{
    tree->Draw(name.c_str());
  }
}

void CBranchManager::Draw(int id)
{
  Draw(id, -1, -1, (TCut*)0);
}

void CBranchManager::Draw(int idx, int idy, int idz, const TCut* selection)
{
  std::string xname, yname, zname;
  
  if(idx >= 0){
    xname = GetName(idx);
    if(xname.empty()) return;
    if(idy >= 0){
      yname = GetName(idy);
      if(yname.empty()) return;
      if(idz >= 0){      
	zname = GetName(idz);
	if(zname.empty()) return;
	Draw(xname + ':' + yname + ':' + zname, selection);	
      }else{
	Draw(xname + ':' + yname, selection);	
      }      
    }else{
      Draw(xname , selection);	
    }      
    CdequeIdNameTitle::iterator itx = FindById(idx);
    if(!Check(itx)){
      std::ostringstream os;
      os << FunctionName("Draw") << "never to come here!!";
      NGlobal::Message(os.str(), kMID);

      return;
    }
    SetCurrent(itx);
  }
}

void CBranchManager::Draw(int idx, int idy, const TCut* selection)
{
  Draw(idx, idy, -1, selection);
}

void CBranchManager::Draw(int idx, const TCut* selection)
{
  Draw(idx, -1, -1, selection);
}

std::string CBranchManager::FunctionName(std::string functionname) const
{
  return "[CBranchManager::" + functionname + "]$ ";
}
