#include "TROOT.h" 
#include "TTree.h"

#include <iostream>
#include <string>
#include <sstream>

#include "CTreeManager.hh"
#include "CBranchManager.hh"
#include "NGlobal.hh"

TTree* CTreeManager::GetTree(std::string name)
{
  TObject* obj = gROOT->FindObjectAny(name.c_str());

  if(obj && obj->InheritsFrom("TTree")) return (TTree*)obj;

  std::ostringstream os;
  os << FunctionName("GetTree") << name << " dosn't exist";
  NGlobal::Message(os.str(), kMID);

  return 0;
}

TTree* CTreeManager::GetTree(int id)
{
  std::string name = GetName(id);

  if(!Check(name)) return 0;

  return GetTree(name);
}

TTree* CTreeManager::GetTree()
{
  std::string name = GetName();  

  if(!Check(name)) return 0;

  return GetTree(name);
}

TTree* CTreeManager::GetTreeTop()
{
  CdequeIdNameTitle::iterator it = Begin();
  if(!Check(it)) return 0;

  TTree* tree = GetTree(it->second);
  if(!Check(tree)) return 0;

  return tree;
}

void CTreeManager::Push(std::string name, std::string title)
{
  IArrayManager::Push(name, title);
  BeFriends();
  TreeToBranch();
}

void CTreeManager::Draw(int)
{
  /* nothing */;
}

void CTreeManager::Write()
{
  CdequeIdNameTitle::iterator it = Begin();
  while(it != End()){
    TTree* tree = GetTree(it->second);
    if(!Check(tree)) return;
    tree->Write();
    ++it;
  }    

  std::ostringstream os;
  os << FunctionName("Write") << "write trees";
  NGlobal::Message(os.str(), kLOW);
}

void CTreeManager::BeFriends()
{
  TTree* toptree = GetTreeTop();
  if(!Check(toptree)) return;

  CdequeIdNameTitle::iterator it = Begin();
  while(it != End()){
    TTree* friendtree = GetTree(it->second);
    if(!Check(friendtree)) return;

    if(friendtree != toptree){
      toptree->AddFriend(friendtree);
    }
    ++it;
  }    

  std::ostringstream os;
  os << FunctionName("BeFriends") << "all trees be friends of " << toptree->GetName();
  NGlobal::Message(os.str(), kMID);
}

void CTreeManager::TreeToBranch()
{
  TTree* tree = GetTree();
  if(!Check(tree)) return;

  NAlias::branchmanager->Push(tree);

//  CdequeIdNameTitle::iterator it = Begin();
//  while(it != End()){
//    TTree* tree = GetTree(it->second);
//    NAlias::branchmanager->Push(tree);
//    ++it;
//  }    
}

std::string CTreeManager::FunctionName(std::string functionname) const
{
  return "[CTreeManager::" + functionname + "]$ ";
}

