#include "TROOT.h"

#include <iostream>
#include <iomanip>
#include <sstream>
#include <string>

#include "IArrayManager.hh"
#include "CCanvasManager.hh"
#include "NGlobal.hh"

void IArrayManager::Insert(std::string name, std::string title, int order, int id)
{
  if(!Check(name)){
    std::ostringstream os;
    os << FunctionName("Insert") << "empty name is not allowed";
    NGlobal::Message(os.str(), kMID);

    return;
  } 

  if(Check(id, kDEBUG)) SetNextId(id);

  if(ExistName(name)){
    std::ostringstream os;
    os << FunctionName("Insert") << name << " exist. better to use another name";
    NGlobal::Message(os.str(), kHIGH);
  }    

  if(Check(order, kDEBUG)){
    CdequeIdNameTitle::iterator it = FindById(order);
    std::ostringstream os;
    os << FunctionName("Insert") << "insert:" << name
       << " after ID:" << order;
    NGlobal::Message(os.str(), kMID);

    it = idnametitle.insert(++it, CIdNameTitle(GetNextId(), name, title));
    SetCurrent(it);
  }else{
    std::ostringstream os;
    os << FunctionName("Insert") << "push:" << name;
    NGlobal::Message(os.str(), kMID);

    idnametitle.push_back(CIdNameTitle(GetNextId(), name, title));
    SetCurrent(--End());
  }    
  PushNextId();
}

void IArrayManager::InsertAfterCurrent(std::string name, std::string title)
{
  int id = GetId();
  if(!Check(id)) return;

  Insert(name, title, id, -1);
}

void IArrayManager::Push(std::string name, std::string title)
{
  Insert(name, title, -1);
}

CdequeIdNameTitle::iterator IArrayManager::Erase(int id)
{
  CdequeIdNameTitle::iterator it = FindById(id);
  if(!Check(it)) return End();

  it = idnametitle.erase(it);
  SetCurrent(it);

  std::ostringstream os;
  os << FunctionName("Erase") << "erase ID:" << id;
  NGlobal::Message(os.str(), kMID);

  return it;
}

CdequeIdNameTitle::iterator IArrayManager::Erase(std::string name)
{
  int id = GetId(name);
  if(!Check(id)) return End();

  return Erase(id);
}

CdequeIdNameTitle::iterator IArrayManager::Erase()
{
  int id = GetId();
  if(!Check(id)) return End();

  return Erase(id);
}

CdequeIdNameTitle::iterator IArrayManager::Erase(int id_begin, int id_end)
{
  CdequeIdNameTitle::iterator it_begin = FindById(id_begin);
  CdequeIdNameTitle::iterator it_end = FindById(id_end);

  if(!Check(it_begin)) return End();
  if(!Check(it_end)) return End();

  CdequeIdNameTitle::iterator it = it_begin;
  while(it != it_end){
    it = Erase(it->first);
    if(it == End()){
      it = Begin();
    }
    it_end = FindById(id_end);
    if(!Check(it_end)) return End();
  }
  return Erase(it_end->first);
}

CdequeIdNameTitle::iterator IArrayManager::GetCurrent()
{
  if(!Check()) return End();
  
  return current;
}

CdequeIdNameTitle::iterator IArrayManager::SetCurrent(CdequeIdNameTitle::iterator it)
{
  return current = it;
}

int IArrayManager::GetId(std::string name)
{
  CdequeIdNameTitle::iterator it = FindByName(name);
  if(!Check(it)) return -1;

  return it->first;
}

int IArrayManager::GetId()
{
  CdequeIdNameTitle::iterator it = GetCurrent();
  if(!Check(it)) return -1;

  return it->first;
}

std::string IArrayManager::GetName(int id)
{
  CdequeIdNameTitle::iterator it = FindById(id);
  if(!Check(it)) return "";

  return it->second;
}

std::string IArrayManager::GetName(std::string title)
{
  CdequeIdNameTitle::iterator it = FindByTitle(title);
  if(!Check(it)) return "";

  return it->second;
}

std::string IArrayManager::GetName()
{
  CdequeIdNameTitle::iterator it = GetCurrent();
  if(!Check(it)) return "";

  return it->second;
}

std::string IArrayManager::GetTitle(int id)
{
  CdequeIdNameTitle::iterator it = FindById(id);
  if(!Check(it)) return "";

  return it->third;
}

std::string IArrayManager::GetTitle(std::string name)
{
  CdequeIdNameTitle::iterator it = FindByName(name);
  if(!Check(it)) return "";

  return it->third;
}

std::string IArrayManager::GetTitle()
{
  CdequeIdNameTitle::iterator it = GetCurrent();
  if(!Check(it)) return "";

  return it->third;
}

void IArrayManager::DrawSequential(int id_begin, int id_end)
{
  CdequeIdNameTitle::iterator it_begin = FindById(id_begin);
  CdequeIdNameTitle::iterator it_end = FindById(id_end);

  if(!Check(it_begin)) return;
  if(!Check(it_end)) return;

  int count = 1;
  CdequeIdNameTitle::iterator it = it_begin;
  while(it != it_end){
    ++count;
    ++it;
    if(it == End()){
      it = Begin();
    }
  }

  double diva = 1.6;
  double divb = 1.1;
  while(count > (int)diva * (int)divb){
    diva += 0.5;
    divb += 0.5;
  }
  NAlias::canvasmanager->Divide((int)diva, (int)divb);

  it = it_begin;
  while(it != it_end){
    NAlias::canvasmanager->NextSubpad();
    Draw(it->first);
    ++it;
    if(it == End()){
      it = Begin();
    }
  }    
  NAlias::canvasmanager->NextSubpad();
  Draw(it->first);
}

void IArrayManager::DrawCurrent()
{
  CdequeIdNameTitle::iterator it = GetCurrent();
  if(!Check(it)) return;

  Draw(it->first);
}

void IArrayManager::DrawPrevious()
{
  CdequeIdNameTitle::iterator it = GetCurrent();
  if(!Check(it)) return;

  if(it == Begin()){
    std::ostringstream os;
    os << FunctionName("DrawPrevious") << "beginning of " << listname << ", goto end of list";
    NGlobal::Message(os.str(), kLOW);

    it = End();
  }
  
  --it;
  Draw(it->first);
  SetCurrent(it);
}

void IArrayManager::DrawNext()
{
  CdequeIdNameTitle::iterator it = GetCurrent();
  if(!Check(it)) return;

  ++it;

  if(it == End()){
    std::ostringstream os;
    os << FunctionName("DrawNext") << "end of " << listname << ", goto beginning of list";
    NGlobal::Message(os.str(), kLOW);

    it = Begin();
  }
  
  Draw(it->first);
  SetCurrent(it);
}

void IArrayManager::Clear()
{
  fList->Delete();
  idnametitle.clear();

  SetCurrent(End());
  ResetNextId();

  std::ostringstream os;
  os << FunctionName("Clear") << "cleared " << listname;
  NGlobal::Message(os.str(), kMID);
}

void IArrayManager::PrintIdName()
{
  CdequeIdNameTitle::iterator it = Begin();
  if(!Check(it)) return;

  int count = 0;

  std::ostringstream os;
  os << FunctionName("PrintIdName") << "print " << listname;
  NGlobal::Message(os.str(), kLOW);

  std::cout << "\n"
	    << "    " << Word(0)
	    << "  Order "
	    << "    ID  "
	    << "    Name                          "
	    << std::endl;
  while(it != End()){
    int wordflag(0);
    if(it == GetCurrent()){
      wordflag = 1;
    }
    std::cout << "    " << Word(wordflag)
	      << std::setw(8)  << std::right << count++
	      << std::setw(8)  << std::right << it->first
	      << "      " << std::left  << it->second
	      << std::endl;
    ++it;
  }

  std::cout << std::endl;  
}

void IArrayManager::PrintIdNameTitle()
{
  if(!Check()) return;

  CdequeIdNameTitle::iterator it = Begin();
  int count = 0;

  std::ostringstream os;
  os << FunctionName("PrintIdName") << "print " << listname << "\n";
  NGlobal::Message(os.str(), kLOW);

  std::cout << "\n"
	    << "    " << Word(0)
	    << "  Order "
	    << "    ID  "
	    << "    Name                          "
	    << "ParentName"
	    << std::endl;
  while(it != End()){
    int wordflag(0);
    if(it == GetCurrent()){
      wordflag = 1;
    }
    std::cout << "    " << Word(wordflag)
	      << std::setw(8)  << std::right << count++
	      << std::setw(8)  << std::right << it->first
	      << "      " << std::setw(26) << std::left  << it->second
	      << "    " << std::left  << it->third
	      << std::endl;
    ++it;
  }

  std::cout << std::endl;  
}

CdequeIdNameTitle::iterator IArrayManager::FindById(int id)
{
  CdequeIdNameTitle::iterator it = idnametitle.FindF(id);
  if(!Check(it)){
    std::ostringstream os;
    os << FunctionName("FindById") << "\nID : " << id << " doesn't exist\n";
    NGlobal::Message(os.str(), kMID);
  }

  return it;
}

CdequeIdNameTitle::iterator IArrayManager::FindByName(std::string name)
{
  CdequeIdNameTitle::iterator it = idnametitle.FindS(name);
  if(!Check(it)){
    std::ostringstream os;
    os << FunctionName("FindByName") << "\nName : " << name << " doesn't exist\n";
    NGlobal::Message(os.str(), kMID);
  }

  return it;
}

CdequeIdNameTitle::iterator IArrayManager::FindByTitle(std::string title)
{
  CdequeIdNameTitle::iterator it = idnametitle.FindT(title);
  if(!Check(it)){
    std::ostringstream os;
    os << FunctionName("FindByTitle") << "\nTitle : " << title << " doesn't exist\n";
    NGlobal::Message(os.str(), kMID);
  }

  return it;
}

bool IArrayManager::ExistId(int id)
{
  CdequeIdNameTitle::iterator it = idnametitle.FindF(id);
  if(!Check(it, kDEBUG)){
    return false;
  }else{
    return true;
  }
}

bool IArrayManager::ExistName(std::string name)
{
  CdequeIdNameTitle::iterator it = idnametitle.FindS(name);
  if(!Check(it, kDEBUG)){
    return false;
  }else{
    return true;
  }
}

bool IArrayManager::ExistTitle(std::string title)
{
  CdequeIdNameTitle::iterator it = idnametitle.FindT(title);
  if(!Check(it, kDEBUG)){
    return false;
  }else{
    return true;
  }
}

int IArrayManager::GetNextId() const
{
  return nextid;
}

int IArrayManager::SetNextId(int id)
{
  if(nextid > id){
    std::ostringstream os;
    os << FunctionName("SetNextId") << "cannot set ID less than " << nextid;
    NGlobal::Message(os.str(), kMID);

    return -1;
  }else{
    nextid = id;
    return nextid;
  }
}

void IArrayManager::PushNextId()
{
  ++nextid;
}

void IArrayManager::ResetNextId()
{
  nextid = 0;
}

CdequeIdNameTitle::size_type IArrayManager::Size()
{
  return idnametitle.size();
}

CdequeIdNameTitle::iterator IArrayManager::Begin()
{
  return idnametitle.begin();
}

CdequeIdNameTitle::iterator IArrayManager::End()
{
  return idnametitle.end();
}

CdequeIdNameTitle::reverse_iterator IArrayManager::RBegin()
{
  return idnametitle.rbegin();
}

CdequeIdNameTitle::reverse_iterator IArrayManager::REnd()
{
  return idnametitle.rend();
}

bool IArrayManager::Check(EMessageLevel level) const
{
  if(current == idnametitle.end()){
    std::ostringstream os;
    os << FunctionName("Check") << "no contents";
    NGlobal::Message(os.str(), level);

    return false;
  }else{
    return true;
  }
}

bool IArrayManager::Check(int id, EMessageLevel level) const
{
  if(id < 0){
    std::ostringstream os;
    os << FunctionName("Check") << "not valid id";
    NGlobal::Message(os.str(), level);

    return false;
  }else{
    return true;
  }
}

bool IArrayManager::Check(std::string str, EMessageLevel level) const
{
  if(str.empty()){
    std::ostringstream os;
    os << FunctionName("Check") << "not valid string";
    NGlobal::Message(os.str(), level);

    return false;
  }else{
    return true;
  }
}

bool IArrayManager::Check(CdequeIdNameTitle::iterator it, EMessageLevel level) const
{
  if(it == idnametitle.end()){
    std::ostringstream os;
    os << FunctionName("Check") << "not valid iterator";
    NGlobal::Message(os.str(), level);

    return false;
  }else{
    return true;
  }
}

std::string IArrayManager::Word(int wordflag)
{
  std::string space;
  for(std::string::size_type i = 0; i < defword.size(); ++i){
    space += " ";
  }

  std::string word;
  if(wordflag == 0){
    word = space;
  }else{
    word = defword; 
  }

  return word;
}
