
#include <cmath>
#include <cstring>

#include "FFunction.hh"
#include "CFold.hh"
#include "TF1.h"

ClassImp(CFold)

CFold::CFold(TF1* fold)
{
  fSigma = 1;
  fStep = 0.01;

  fFold = fold;

  Double_t xmin, xmax;
  fFold->GetRange(xmin, xmax);
  Int_t npar = fFold->GetNumberFreeParameters();
  fFolded = new TF1("cfold", this, &CFold::Evaluate, xmin, xmax, npar);
  fFolded->SetParameters(fFold->GetParameters());

  fGaussian = new TF1("gaussian", Gaussian, xmin, xmax, 2);
  fPoisson  = new TF1("poisson",  Poisson,  xmin, xmax, 3);
}

CFold::~CFold()
{
  delete fPoisson;
  delete fGaussian;
  delete fFolded;
}

void CFold::SetSigma(Double_t sigma)
{
  fSigma = sigma;
}

void CFold::SetStep(Double_t step)
{
  fStep = step;
}

TF1* CFold::GetFoldedFunction()
{
  return fFolded;
}

Double_t CFold::Evaluate(Double_t* x, Double_t* par)
{
  fFold->SetParameters(par);

  Double_t xmin, xmax;
  fFold->GetRange(xmin, xmax);
  Double_t sum = 0;
  for(Double_t xx = xmin; xx < xmax; xx += fStep){

//    Double_t sigma = fSigma*sqrt(std::abs(xx));
//    fGaussian->SetParameters(xx, sigma);
//    Double_t g = fGaussian->Eval(x[0]);

    Double_t k = -50.6*xx+133;
    Double_t lambda = 110*xx+2.34;
    fPoisson->SetParameters(1, k, lambda);
    Double_t g = fPoisson->Eval(x[0]);

    Double_t f = fFold->Eval(xx);

    sum += f*g*fStep;
  }

  return sum;
}

