
#include <iostream>
#include <deque>
#include <fstream>
#include <cmath>

#include "TF1.h"

#include "CMapFunction1D.hh"
#include "FFunction.hh"

ClassImp(CMapFunction1D)

CMapFunction1D::CMapFunction1D() : CMapFunction()
{;}

CMapFunction1D::~CMapFunction1D()
{;}

void CMapFunction1D::SetFile(std::string filename)
{
  std::ifstream ifs(filename.c_str());
  Double_t f,x;
  std::deque<Double_t> df,dx;
  while(!ifs.eof()){
    ifs >> x >> f;
    df.push_back(f);
    dx.push_back(x);
  }
  SetMap(&df,&dx);

  std::deque<Double_t>::const_iterator itx = fX.begin();
  Double_t xmin=*itx;
  Double_t xmax=*itx;
  while(itx != fX.end()){
    if(*itx < xmin) xmin = *itx;
    if(*itx > xmax) xmax = *itx;
    ++itx;
  }
  fMapFunction = new TF1("mapfunction1d", this,
			 static_cast<Double_t (CMapFunction1D::*)(Double_t*, Double_t*)>
			 (&CMapFunction1D::MapFunction),
			 xmin, xmax, 0);
}

Double_t CMapFunction1D::MapFunction(Double_t xx)
{
  Double_t result;

  std::deque<Double_t>::const_iterator itf = fF.begin();
  std::deque<Double_t>::const_iterator itx = fX.begin();
  Double_t lx=*itx;
  Double_t lf=*itf;
  Double_t lxtemp;
  Double_t hx=*itx;
  Double_t hf=*itf;
  Double_t hxtemp;
  while(1){
    lxtemp = xx-*itx;
    if((xx < lx) || (lxtemp >= 0 && lxtemp < xx-lx)){
      lx = *itx;
      lf = *itf;
    }

    hxtemp = *itx-xx;
    if((xx >= hx) || (hxtemp > 0 && hxtemp < hx-xx)){
      hx = *itx;
      hf = *itf;
    }

    ++itf;
    ++itx;
    if(itf == fF.end() || itx == fX.end()) break;
  }
  if(xx < lx || xx >= hx){
    std::cout << xx << " is out of range" << std::endl;
    result = 0;
  }else{
    result = (xx-lx)*(hf-lf)/(hx-lx)+lf;
  }

  return result;
}

Double_t CMapFunction1D::MapFunction(Double_t* x, Double_t* /*par*/)
{
  Double_t xx = x[0];

  return MapFunction(xx);
}

Double_t CMapFunction1D::VRMF1D(Double_t xx, Double_t k, Double_t a, Double_t r0,
			      Double_t Eb, Double_t ma, Double_t mb)
{
  return k*VirtualResonance(xx, a, r0, Eb, ma, mb)*MapFunction(xx);
}

Double_t CMapFunction1D::VRMF1D(Double_t* x, Double_t* par)
{
  Double_t xx = x[0];
  Double_t k = par[0];
  Double_t a = par[1];
  Double_t r0 = par[2];
  Double_t Eb = par[3];
  Double_t ma = par[4];
  Double_t mb = par[5];

  return VRMF1D(xx, k, a, r0, Eb, ma, mb);
}

Double_t CMapFunction1D::BW2MF1D(Double_t xx, Double_t k,
				 Double_t k1, Double_t er1, Double_t gamma1,
				 Double_t er2, Double_t gamma2)
{
  return k*(k1*BrietWigner(xx, er1, gamma1)+BrietWigner(xx, er2, gamma2))
    *MapFunction(xx);
}

Double_t CMapFunction1D::BW2MF1D(Double_t* x, Double_t* par)
{
  Double_t xx = x[0];
  Double_t k = par[0];
  Double_t k1 = par[1];
  Double_t er1 = par[2];
  Double_t gamma1 = par[3];
  Double_t er2 = par[4];
  Double_t gamma2 = par[5];

  return BW2MF1D(xx, k, k1, er1, gamma1, er2, gamma2);
}

Double_t CMapFunction1D::VRBW2MF1D(Double_t xx, Double_t k,
				   Double_t a, Double_t r0, Double_t Eb, Double_t ma, Double_t mb,
				   Double_t k1, Double_t er1, Double_t gamma1,
				   Double_t k2, Double_t er2, Double_t gamma2)
{
  return k*(VirtualResonance(xx, a, r0, Eb, ma, mb)
	    +k1*BrietWigner(xx, er1, gamma1)
	    +k2*BrietWigner(xx, er2, gamma2))
    *MapFunction(xx);
}

Double_t CMapFunction1D::VRBW2MF1D(Double_t* x, Double_t* par)
{
  Double_t xx = x[0];
  Double_t k = par[0];

  Double_t a = par[1];
  Double_t r0 = par[2];
  Double_t Eb = par[3];
  Double_t ma = par[4];
  Double_t mb = par[5];

  Double_t k1 = par[6];
  Double_t er1 = par[7];
  Double_t gamma1 = par[8];

  Double_t k2 = par[9];
  Double_t er2 = par[10];
  Double_t gamma2 = par[11];

  return VRBW2MF1D(xx, k, a, r0, Eb, ma, mb, k1, er1, gamma1, k2, er2, gamma2);

}
